package com.zipcodewilmington.gildedrose;

import org.junit.Assert;
import org.junit.Test;


public class InventoryTest {

    Item genericItem = new Item("Generic Item", 7, 25);
    Item agedbrietest = new Item("Aged Brie", 7, 4);
    Item backstagepasstest = new Item("Backstage passes to a TAFKAL80ETC concert", 13, 45);
    Item sulfurastest = new Item("Sulfuras, Hand of Ragnaros", 7, 20);

    Item[] inventoryList = {genericItem, agedbrietest, backstagepasstest, sulfurastest};

    @Test
    public void updateQualityTest() {
        Inventory test = new Inventory(inventoryList);
        int before = inventoryList[0].getQuality();
        test.updateQuality();
        int after = inventoryList[0].getQuality();
        //int expected = 5;
        //int actual = agedbrietest.getQuality();
        Assert.assertTrue(before > after);
    }

    @Test
    public void aged_sellInTest() {
        Inventory test = new Inventory(inventoryList);

        test.updateQuality();

        int expected = 6;
        int actual = agedbrietest.getSellIn();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void sulfur_sellInTest() {
        Inventory test = new Inventory(inventoryList);

        test.updateQuality();

        int expected = 7;
        int actual = sulfurastest.getSellIn();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void backstage_sellInTest() {
        Inventory test = new Inventory(inventoryList);

        test.updateQuality();

        int expected = 12;
        int actual = backstagepasstest.getSellIn();
        Assert.assertEquals(expected, actual);
    }

    public void setNameTest() {


        sulfurastest.setName("sulphura");
        String expected = "sulphura";
        String actual = sulfurastest.getName();
        Assert.assertEquals(expected,actual);
    }

    @Test
    public void getNameTest() {


        String expected = "Aged Brie";
        String actual = agedbrietest.getName();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void getQualityTest() {


        int expected = 4;

        int actual = agedbrietest.getQuality();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void setQualityTest() {
        agedbrietest.setQuality(8);


        int expected = 8;
        int actual = agedbrietest.getQuality();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void getSellinTest() {


        int expected = 13;
        int actual = backstagepasstest.getSellIn();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void setSellInTest() {
        sulfurastest.setSellIn(4);


        int expected = 4;
        int actual = sulfurastest.getSellIn();
        Assert.assertEquals(expected, actual);
    }

    @Test
    public void sellByDatePassedTest() {
        Inventory test = new Inventory(inventoryList);
        genericItem.setSellIn(1);
        test.updateQuality();
        int before = genericItem.getQuality();
        test.updateQuality();
        int after = genericItem.getQuality();
        Assert.assertTrue(after == (before - 2));

    }

    @Test
    public void neverNegativeTest() {
        Inventory test = new Inventory(inventoryList);
        agedbrietest.setQuality(0);
        int before = agedbrietest.getQuality();
        test.updateQuality();
        int after = agedbrietest.getQuality();
        Assert.assertTrue(after > 0);
    }

    @Test
    public void agedBrieImprovesTest() {
        Inventory test = new Inventory(inventoryList);
        int before = agedbrietest.getQuality();
        test.updateQuality();
        int after = agedbrietest.getQuality();
        Assert.assertTrue(after > before);
    }

    @Test
    public void qualityNeverMoreThan50Test() {
        Inventory test = new Inventory(inventoryList);
        agedbrietest.setSellIn(0);
        agedbrietest.setQuality(47);
        int before = agedbrietest.getQuality();
        test.updateQuality();
        test.updateQuality();
        test.updateQuality();
        test.updateQuality();
        test.updateQuality();
        int after = agedbrietest.getQuality();
        Assert.assertTrue(after == 50);
    }

    @Test
    public void sulfurasNeverDegradesTest() {
        Inventory test = new Inventory(inventoryList);
        int before = sulfurastest.getQuality();
        test.updateQuality();
        int after = sulfurastest.getQuality();
        Assert.assertTrue(before == after);
    }

    @Test
    public void sulfurasNeverSellOutTest() {
        Inventory test = new Inventory(inventoryList);

        int before = sulfurastest.getSellIn();
        test.updateQuality();

        int after = sulfurastest.getSellIn();
        Assert.assertTrue(after == before);
    }

    @Test
    public void backStagePassQualityTest1() {
        Inventory test = new Inventory(inventoryList);
        backstagepasstest.setSellIn(9);
        int before = backstagepasstest.getQuality();
        int temp = backstagepasstest.getSellIn();
        test.updateQuality();
        int after = 0;
        if (temp > 10) {
            after=before+1;
        }

        if (temp <= 10) {
            after = before + 2;
        }

        int expected = after;
        int actual = backstagepasstest.getQuality();

        Assert.assertEquals(expected,actual);
    }

    @Test
    public void backStagePassQualityTest2() {
        Inventory test = new Inventory(inventoryList);
        backstagepasstest.setSellIn(4);
        int before = backstagepasstest.getQuality();
        int temp = backstagepasstest.getSellIn();
        test.updateQuality();
        int after = 0;
        if (temp > 10) {
            after=before+1;
        }

        if (temp <= 5) {
            after = before + 3;
        }

        int expected = after;
        int actual = backstagepasstest.getQuality();

        Assert.assertEquals(expected,actual);
    }

    @Test
    public void backStagePassQualityTest3() {
        Inventory test = new Inventory(inventoryList);
        backstagepasstest.setSellIn(0);
        int before = backstagepasstest.getQuality();
        int temp = backstagepasstest.getSellIn();
        test.updateQuality();
        int after = 0;
        if (temp > 10) {
            after=before+1;
        }

        if (temp <= 0) {
            after = 0;
        }

        int expected = after;
        int actual = backstagepasstest.getQuality();

        Assert.assertEquals(expected,actual);
    }


}



